@@ -41,14 +41,21 @@ from math import (
 
 from typing import (
     Any as _Any,
+    Dict as _Dict,
     Iterable as _Iterable,
     List as _List,
     Optional as _Optional,
+    Tuple as _Tuple,
     Union as _Union
 )
 
 # Internal
 
+from pydtmc.custom_types import (
+    onumeric as _onumeric,
+    tnumeric as _tnumeric
+)
+
 from pydtmc.decorators import (
     alias as _alias,
     aliased as _aliased,
@@ -56,57 +63,28 @@ from pydtmc.decorators import (
 )
 
 from pydtmc.exceptions import (
-    ValidationError
+    ValidationError as _ValidationError
 )
 
 from pydtmc.validation import (
     validate_boolean as _validate_boolean,
+    validate_dictionary as _validate_dictionary,
     validate_enumerator as _validate_enumerator,
     validate_hyperparameter as _validate_hyperparameter,
-    validate_integer_non_negative as _validate_integer_non_negative,
-    validate_integer_positive as _validate_integer_positive,
+    validate_integer as _validate_integer,
     validate_mask as _validate_mask,
     validate_matrix as _validate_matrix,
     validate_rewards as _validate_rewards,
     validate_state as _validate_state,
-    validate_states as _validate_states,
     validate_state_names as _validate_state_names,
+    validate_states as _validate_states,
+    validate_status as _validate_status,
     validate_transition_matrix as _validate_transition_matrix,
     validate_transition_matrix_size as _validate_transition_matrix_size,
     validate_vector as _validate_vector
 )
 
 
-##############
-# ATTRIBUTES #
-##############
-
-
-_tnumeric = _Union[_Iterable, _np.ndarray]
-
-# noinspection PyBroadException
-try:
-    import pandas as _pd
-    _tnumeric = _Union[_tnumeric, _pd.DataFrame, _pd.Series]
-except Exception:
-    pass
-
-# noinspection PyBroadException
-try:
-    import scipy.sparse as _sps
-    _tnumeric = _Union[_tnumeric, _sps.bsr.bsr_matrix]
-    _tnumeric = _Union[_tnumeric, _sps.coo.coo_matrix]
-    _tnumeric = _Union[_tnumeric, _sps.csc.csc_matrix]
-    _tnumeric = _Union[_tnumeric, _sps.csr.csr_matrix]
-    _tnumeric = _Union[_tnumeric, _sps.dia.dia_matrix]
-    _tnumeric = _Union[_tnumeric, _sps.dok.dok_matrix]
-    _tnumeric = _Union[_tnumeric, _sps.lil.lil_matrix]
-except Exception:
-    pass
-
-_onumeric = _Optional[_tnumeric]
-
-
 ###########
 # CLASSES #
 ###########
@@ -119,7 +97,7 @@ class MarkovChain(object):
     Defines a Markov chain with given transition matrix and state names.
 
     :param p: the transition matrix.
-    :param states: the name of each state (by default, an increasing sequence of integers starting at 1).
+    :param states: the name of each state (if omitted, an increasing sequence of integers starting at 1).
     :raises ValidationError: if any input argument is not compliant.
     """
 
@@ -141,7 +119,7 @@ class MarkovChain(object):
 
             except Exception as e:
                 argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-                raise ValidationError(str(e).replace('@arg@', argument)) from None
+                raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         self._digraph: _nx.DiGraph = _nx.DiGraph(p)
         self._p: _np.ndarray = p
@@ -151,15 +129,17 @@ class MarkovChain(object):
     # noinspection PyListCreation
     def __str__(self) -> str:
 
-        indentation = (' ' * 2)
-
         lines = []
         lines.append('')
         lines.append('DISCRETE-TIME MARKOV CHAIN')
-        lines.append(f'{indentation}ABSORBING:   {("YES" if self.is_absorbing else "NO")}')
-        lines.append(f'{indentation}APERIODIC:   {("YES" if self.is_aperiodic else "NO (" + str(self.period) + ")")}')
-        lines.append(f'{indentation}IRREDUCIBLE: {("YES" if self.is_irreducible else "NO")}')
-        lines.append(f'{indentation}ERGODIC:     {("YES" if self.is_ergodic else "NO")}')
+        lines.append(f' SIZE:         {self._size:d}')
+        lines.append(f' CLASSES:      {len(self.communicating_classes):d}')
+        lines.append(f'  - RECURRENT: {len(self.recurrent_classes):d}')
+        lines.append(f'  - TRANSIENT: {len(self.transient_classes):d}')
+        lines.append(f' ABSORBING:    {("YES" if self.is_absorbing else "NO")}')
+        lines.append(f' APERIODIC:    {("YES" if self.is_aperiodic else "NO (" + str(self.period) + ")")}')
+        lines.append(f' IRREDUCIBLE:  {("YES" if self.is_irreducible else "NO")}')
+        lines.append(f' ERGODIC:      {("YES" if self.is_ergodic else "NO")}')
         lines.append('')
 
         return '\n'.join(lines)
@@ -169,15 +149,15 @@ class MarkovChain(object):
 
         return [i for i in range(self._size) if _np.isclose(self._p[i, i], 1.0)]
 
+    @_cachedproperty
+    def _classes_indices(self) -> _List[_List[int]]:
+
+        return [sorted([index for index in component]) for component in _nx.strongly_connected_components(self._digraph)]
+
     @_cachedproperty
     def _communicating_classes_indices(self) -> _List[_List[int]]:
 
-        indices = list()
-
-        for sccs in _nx.strongly_connected_components(self._digraph):
-            indices.append(sorted(list(sccs)))
-
-        return sorted(indices, key=lambda x: (-len(x), x[0]))
+        return sorted(self._classes_indices, key=lambda x: (-len(x), x[0]))
 
     @_cachedproperty
     def _cyclic_classes_indices(self) -> _List[_List[int]]:
@@ -239,18 +219,7 @@ class MarkovChain(object):
     @_cachedproperty
     def _recurrent_classes_indices(self) -> _List[_List[int]]:
 
-        indices = list()
-
-        for sccs in _nx.strongly_connected_components(self._digraph):
-
-            sccs_reachable = sccs.copy()
-
-            for scc_reachable in sccs_reachable:
-                spl = _nx.shortest_path_length(self._digraph, scc_reachable).keys()
-                sccs_reachable = sccs_reachable.union(spl)
-
-            if (sccs_reachable - sccs) == set():
-                indices.append(sorted(list(sccs)))
+        indices = [index for index in self._classes_indices if index not in self._transient_classes_indices]
 
         return sorted(indices, key=lambda x: (-len(x), x[0]))
 
@@ -265,7 +234,7 @@ class MarkovChain(object):
         if not self.is_ergodic:
             return None
 
-        values, _ = _npl.eig(self._p)
+        values = _npl.eigvals(self._p)
         values_abs = _np.sort(_np.abs(values))
         values_ct1 = _np.isclose(values_abs, 1.0)
 
@@ -287,18 +256,8 @@ class MarkovChain(object):
     @_cachedproperty
     def _transient_classes_indices(self) -> _List[_List[int]]:
 
-        indices = list()
-
-        for sccs in _nx.strongly_connected_components(self._digraph):
-
-            sccs_reachable = sccs.copy()
-
-            for scc_reachable in sccs_reachable:
-                spl = _nx.shortest_path_length(self._digraph, scc_reachable).keys()
-                sccs_reachable = sccs_reachable.union(spl)
-
-            if (sccs_reachable - sccs) != set():
-                indices.append(sorted(list(sccs)))
+        edges = set([edge1 for (edge1, edge2) in _nx.condensation(self._digraph).edges])
+        indices = [self._classes_indices[edge] for edge in edges]
 
         return sorted(indices, key=lambda x: (-len(x), x[0]))
 
@@ -414,6 +373,15 @@ class MarkovChain(object):
 
         return [*map(self._states.__getitem__, self._cyclic_states_indices)]
 
+    @_cachedproperty
+    def determinant(self) -> float:
+
+        """
+        A property representing the determinant the transition matrix of the Markov chain.
+        """
+
+        return _npl.det(self._p)
+
     @_cachedproperty
     def entropy_rate(self) -> _Optional[float]:
 
@@ -546,6 +514,19 @@ class MarkovChain(object):
 
         return len(self.communicating_classes) == 1
 
+    @_cachedproperty
+    def is_regular(self) -> bool:
+
+        """
+        A property indicating whether the Markov chain is regular.
+        """
+
+        values = _npl.eigvals(self._p)
+        values_abs = _np.sort(_np.abs(values))
+        values_ct1 = _np.isclose(values_abs, 1.0)
+
+        return values_ct1[0] and not any(values_ct1[1:])
+
     @_cachedproperty
     def is_reversible(self) -> bool:
 
@@ -575,8 +556,8 @@ class MarkovChain(object):
 
         return _np.asscalar(_np.trace(n))
 
-    @_alias('mfpt')
     @_cachedproperty
+    @_alias('mfpt')
     def mean_first_passage_times(self) -> _Optional[_np.ndarray]:
 
         """
@@ -604,12 +585,10 @@ class MarkovChain(object):
         A property representing the mixing rate of the Markov chain. If the *SLEM* (second largest eigenvalue modulus) cannot be computed, then None is returned.
         """
 
-        slem = self._slem
-
-        if slem is None:
+        if self._slem is None:
             return None
 
-        return -1.0 / _np.log(slem)
+        return -1.0 / _np.log(self._slem)
 
     @property
     def p(self) -> _np.ndarray:
@@ -666,8 +645,8 @@ class MarkovChain(object):
 
         return periods
 
-    @_alias('stationary_distributions', 'steady_states')
     @_cachedproperty
+    @_alias('stationary_distributions', 'steady_states')
     def pi(self) -> _List[_np.ndarray]:
 
         """
@@ -717,12 +696,10 @@ class MarkovChain(object):
         A property representing the relaxation rate of the Markov chain. If the *SLEM* (second largest eigenvalue modulus) cannot be computed, then None is returned.
         """
 
-        slem = self._slem
-
-        if slem is None:
+        if self._slem is None:
             return None
 
-        return 1.0 / (1.0 - slem)
+        return 1.0 / (1.0 - self._slem)
 
     @property
     def size(self) -> int:
@@ -775,7 +752,7 @@ class MarkovChain(object):
     def are_communicating(self, state1: _Union[int, str], state2: _Union[int, str]) -> bool:
 
         """
-        The method verifies whether the given states are communicating.
+        The method verifies whether the given states of the Markov chain are communicating.
 
         :param state1: the first state.
         :param state2: the second state.
@@ -790,7 +767,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         a1 = self.accessibility_matrix[state1, state2] != 0
         a2 = self.accessibility_matrix[state2, state1] != 0
@@ -801,7 +778,7 @@ class MarkovChain(object):
     def backward_committor_probabilities(self, states1: _Union[int, str, _Iterable[int], _Iterable[str]], states2: _Union[int, str, _Iterable[int], _Iterable[str]]) -> _Optional[_np.ndarray]:
 
         """
-        The method computes the backward committor probabilities between the given subsets of the state space.
+        The method computes the backward committor probabilities between the given subsets of the state space defined by the Markov chain.
 
         | **Aliases:** backward_committor
 
@@ -819,7 +796,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         if not self.is_ergodic:
             return None
@@ -847,7 +824,7 @@ class MarkovChain(object):
     def conditional_probabilities(self, state: _Union[int, str]) -> _np.ndarray:
 
         """
-        The method computes the probabilities, for all the states, conditioned on the process being at a given state.
+        The method computes the probabilities, for all the states of the Markov chain, conditioned on the process being at a given state.
 
         | **Aliases:** conditional_distribution
 
@@ -857,17 +834,19 @@ class MarkovChain(object):
         """
 
         try:
+
             state = _validate_state(state, self._states)
+
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         return self._p[state, :]
 
     def expected_rewards(self, steps: int, rewards: _tnumeric) -> _np.ndarray:
 
         """
-        The method computes the expected rewards after N steps, given the reward value of each state.
+        The method computes the expected rewards of the Markov chain after N steps, given the reward value of each state.
 
         :param steps: the number of steps.
         :param rewards: the reward values.
@@ -878,11 +857,11 @@ class MarkovChain(object):
         try:
 
             rewards = _validate_rewards(rewards, self._size)
-            steps = _validate_integer_positive(steps)
+            steps = _validate_integer(steps, lower_limit=(0, True))
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         original_rewards = rewards.copy()
 
@@ -894,7 +873,7 @@ class MarkovChain(object):
     def expected_transitions(self, steps: int, initial_distribution: _onumeric = None) -> _Optional[_np.ndarray]:
 
         """
-        The method computes the expected number of transitions after N steps, given the initial distribution of the states.
+        The method computes the expected number of transitions performed by the Markov chain after N steps, given the initial distribution of the states.
 
         :param steps: the number of steps.
         :param initial_distribution: the initial distribution of the states (if omitted, the states are assumed to be uniformly distributed).
@@ -904,16 +883,16 @@ class MarkovChain(object):
 
         try:
 
-            steps = _validate_integer_positive(steps)
+            steps = _validate_integer(steps, lower_limit=(0, True))
 
             if initial_distribution is None:
                 initial_distribution = _np.ones(self._size, dtype=float) / self._size
             else:
-                initial_distribution = _validate_vector(initial_distribution, 'stochastic', flex=False, size=self._size)
+                initial_distribution = _validate_vector(initial_distribution, 'stochastic', False, size=self._size)
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         if steps <= self._size:
 
@@ -963,7 +942,7 @@ class MarkovChain(object):
     def forward_committor_probabilities(self, states1: _Union[int, str, _Iterable[int], _Iterable[str]], states2: _Union[int, str, _Iterable[int], _Iterable[str]]) -> _Optional[_np.ndarray]:
 
         """
-        The method computes the forward committor probabilities between the given subsets of the state space.
+        The method computes the forward committor probabilities between the given subsets of the state space defined by the Markov chain.
 
         | **Aliases:** forward_committor
 
@@ -981,7 +960,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         if not self.is_ergodic:
             return None
@@ -1008,7 +987,7 @@ class MarkovChain(object):
     def hitting_probabilities(self, states: _Optional[_Union[int, str, _Iterable[int], _Iterable[str]]] = None) -> _np.ndarray:
 
         """
-        The method computes the hitting probability, for all the states, to the given set of states.
+        The method computes the hitting probability, for all the states of the Markov chain, to the given set of states.
 
         :param states: the set of target states (if omitted, all the states are targeted).
         :return: the hitting probability of each state of the Markov chain.
@@ -1024,7 +1003,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         states = sorted(states)
 
@@ -1048,7 +1027,7 @@ class MarkovChain(object):
     def is_absorbing_state(self, state: _Union[int, str]) -> bool:
 
         """
-        The method verifies whether the given state is absorbing.
+        The method verifies whether the given state of the Markov chain is absorbing.
 
         :param state: the target state.
         :return: True if the state is absorbing, False otherwise.
@@ -1056,10 +1035,12 @@ class MarkovChain(object):
         """
 
         try:
+
             state = _validate_state(state, self._states)
+
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         return state in self._absorbing_states_indices
 
@@ -1081,7 +1062,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         return self.accessibility_matrix[state_origin, state_target] != 0
 
@@ -1096,10 +1077,12 @@ class MarkovChain(object):
         """
 
         try:
+
             state = _validate_state(state, self._states)
+
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         return state in self._cyclic_states_indices
 
@@ -1114,10 +1097,12 @@ class MarkovChain(object):
         """
 
         try:
+
             state = _validate_state(state, self._states)
+
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         return state in self._recurrent_states_indices
 
@@ -1132,10 +1117,12 @@ class MarkovChain(object):
         """
 
         try:
+
             state = _validate_state(state, self._states)
+
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         return state in self._transient_states_indices
 
@@ -1160,7 +1147,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         if not self.is_irreducible:
             return None
@@ -1210,7 +1197,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         states = sorted(states)
 
@@ -1230,7 +1217,7 @@ class MarkovChain(object):
 
         :param initial_distribution: the initial distribution of the states (if omitted, the states are assumed to be uniformly distributed).
         :param jump: the number of steps in each iteration (by default, 1).
-        :param cutoff_type: the type of cutoff to use (either natural or traditional, natural by default).
+        :param cutoff_type: the type of cutoff to use (either natural or traditional; natural by default).
         :return: the mixing time if the Markov chain is *ergodic*, None otherwise.
         :raises ValidationError: if any input argument is not compliant.
         """
@@ -1240,14 +1227,14 @@ class MarkovChain(object):
             if initial_distribution is None:
                 initial_distribution = _np.ones(self._size, dtype=float) / self._size
             else:
-                initial_distribution = _validate_vector(initial_distribution, 'stochastic', flex=False, size=self._size)
+                initial_distribution = _validate_vector(initial_distribution, 'stochastic', False, size=self._size)
 
-            jump = _validate_integer_positive(jump)
+            jump = _validate_integer(jump, lower_limit=(0, True))
             cutoff_type = _validate_enumerator(cutoff_type, ['natural', 'traditional'])
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         if not self.is_ergodic:
             return None
@@ -1290,7 +1277,7 @@ class MarkovChain(object):
 
             rng = MarkovChain._create_rng(seed)
 
-            steps = _validate_integer_positive(steps)
+            steps = _validate_integer(steps, lower_limit=(0, True))
 
             if initial_state is None:
                 initial_state = rng.randint(0, self._size)
@@ -1302,7 +1289,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         prediction = list()
 
@@ -1326,13 +1313,13 @@ class MarkovChain(object):
 
         return prediction
 
-    def redistribute(self, steps: int, initial_distribution: _onumeric = None, include_initial: bool = False, output_last: bool = True) -> _List[_np.ndarray]:
+    def redistribute(self, steps: int, initial_status: _Optional[_Union[int, str, _tnumeric]] = None, include_initial: bool = False, output_last: bool = True) -> _List[_np.ndarray]:
 
         """
         The method simulates a redistribution of states of N steps.
 
         :param steps: the number of steps.
-        :param initial_distribution: the initial distribution of the states (if omitted, the states are assumed to be uniformly distributed).
+        :param initial_status: the initial state or the initial distribution of the states (if omitted, the states are assumed to be uniformly distributed).
         :param include_initial: a boolean indicating whether to include the initial distribution in the output sequence (by default, False).
         :param output_last: a boolean indicating whether to the output only the last distributions (by default, True).
         :return: the sequence of redistributions produced by the simulation.
@@ -1341,26 +1328,26 @@ class MarkovChain(object):
 
         try:
 
-            steps = _validate_integer_positive(steps)
+            steps = _validate_integer(steps, lower_limit=(0, True))
 
-            if initial_distribution is None:
-                initial_distribution = _np.ones(self._size, dtype=float) / self._size
+            if initial_status is None:
+                initial_status = _np.ones(self._size, dtype=float) / self._size
             else:
-                initial_distribution = _validate_vector(initial_distribution, 'stochastic', flex=False, size=self._size)
+                initial_status = _validate_status(initial_status, self._states)
 
             include_initial = _validate_boolean(include_initial)
             output_last = _validate_boolean(output_last)
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         distributions = _np.zeros((steps, self._size), dtype=float)
 
         for i in range(steps):
 
             if i == 0:
-                distributions[i, :] = initial_distribution.dot(self._p)
+                distributions[i, :] = initial_status.dot(self._p)
             else:
                 distributions[i, :] = distributions[i - 1, :].dot(self._p)
 
@@ -1370,7 +1357,7 @@ class MarkovChain(object):
             distributions = distributions[-1:, :]
 
         if include_initial:
-            distributions = _np.vstack((initial_distribution, distributions))
+            distributions = _np.vstack((initial_status, distributions))
 
         return [_np.ravel(x) for x in _np.split(distributions, distributions.shape[0])]
 
@@ -1385,10 +1372,12 @@ class MarkovChain(object):
         """
 
         try:
+
             state = _validate_state(state, self._states)
+
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         if not self.is_irreducible:
             return None
@@ -1436,7 +1425,7 @@ class MarkovChain(object):
     def to_directed_graph(self, multi: bool = True) -> _Union[_nx.DiGraph, _nx.MultiDiGraph]:
 
         """
-        The method returns a directed graph representing the process.
+        The method returns a directed graph representing the Markov chain.
 
         | **Aliases:** to_digraph
 
@@ -1446,10 +1435,12 @@ class MarkovChain(object):
         """
 
         try:
+
             multi = _validate_boolean(multi)
+
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         if multi:
             graph = _nx.MultiDiGraph(self._p)
@@ -1471,10 +1462,12 @@ class MarkovChain(object):
         """
 
         try:
-            inertial_weights = _validate_vector(inertial_weights, 'regular', flex=True, size=self._size)
+
+            inertial_weights = _validate_vector(inertial_weights, 'unconstrained', True, size=self._size)
+
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         p_adjusted = ((1.0 - inertial_weights)[:, _np.newaxis] * self._p) + (_np.eye(self._size) * inertial_weights)
 
@@ -1491,10 +1484,12 @@ class MarkovChain(object):
         """
 
         try:
+
             states = _validate_states(states, self._states, 'subset', True)
+
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         closure = self.adjacency_matrix.copy()
 
@@ -1535,7 +1530,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         return self._p[state_origin, state_target]
 
@@ -1558,7 +1553,7 @@ class MarkovChain(object):
 
             rng = MarkovChain._create_rng(seed)
 
-            steps = _validate_integer_positive(steps)
+            steps = _validate_integer(steps, lower_limit=(0, True))
 
             if initial_state is None:
                 initial_state = rng.randint(0, self._size)
@@ -1574,7 +1569,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         walk = list()
 
@@ -1608,10 +1603,12 @@ class MarkovChain(object):
         """
 
         try:
+
             walk = _validate_states(walk, self._states, 'walk', False)
+
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         p = 0.0
 
@@ -1718,7 +1715,7 @@ class MarkovChain(object):
 
         :param q: the creation probabilities.
         :param p: the annihilation probabilities.
-        :param states: the name of each state (by default, an increasing sequence of integers starting at 1).
+        :param states: the name of each state (if omitted, an increasing sequence of integers starting at 1).
         :return: a Markov chain.
         :raises ValidationError: if any input argument is not compliant.
         :raises ValueError: if q and p have different a size or if the vector resulting from the sum of q and p contains any value greater than one.
@@ -1726,12 +1723,12 @@ class MarkovChain(object):
 
         try:
 
-            p = _validate_vector(p, 'creation')
-            q = _validate_vector(q, 'annihilation')
+            p = _validate_vector(p, 'creation', False)
+            q = _validate_vector(q, 'annihilation', False)
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         if p.shape[0] != q.shape[0]:
             raise ValueError(f'The assets vector and the liabilities vector must have the same size.')
@@ -1750,7 +1747,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         r = 1.0 - q - p
         p = _np.diag(r, k=0) + _np.diag(p[0:-1], k=1) + _np.diag(q[1:], k=-1)
@@ -1784,7 +1781,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         frequencies = _np.zeros((size, size), dtype=float)
 
@@ -1829,7 +1826,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         p_size = len(possible_states)
         p = _np.zeros((p_size, p_size), dtype=int)
@@ -1848,14 +1845,48 @@ class MarkovChain(object):
 
         return MarkovChain(p, possible_states)
 
+    @staticmethod
+    def from_dictionary(d: _Dict[_Tuple[str, str], _Union[float, int]]) -> 'MarkovChain':
+
+        """
+        The method generates a Markov chain from the given dictionary, whose keys represent state pairs and whose values represent transition probabilities.
+
+        :param d: the dictionary to transform into the transition matrix.
+        :raises ValidationError: if any input argument is not compliant.
+        """
+
+        try:
+
+            d = _validate_dictionary(d)
+
+        except Exception as e:
+            argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
+
+        states = sorted(list(set(sum(d.keys(), ()))))
+        size = len(states)
+
+        if size < 2:
+            raise ValueError('The size of the transition matrix defined by the dictionary must be greater than or equal to 2.')
+
+        m = _np.zeros((size, size), dtype=float)
+
+        for transition, probability in d.items():
+            m[states.index(transition[0]), states.index(transition[1])] = probability
+
+        if not _np.allclose(_np.sum(m, axis=1), _np.ones(size)):
+            raise ValueError('The rows of the transition matrix defined by the dictionary must sum to 1.')
+
+        return MarkovChain(m, states)
+
     @staticmethod
     def from_matrix(m: _tnumeric, states: _Optional[_Iterable[str]] = None) -> 'MarkovChain':
 
         """
-        The method generates a Markov chain with the given state names whose transition matrix is obtained through the normalization of the given matrix.
+        The method generates a Markov chain with the given state names, whose transition matrix is obtained through the normalization of the given matrix.
 
         :param m: the matrix to transform into the transition matrix.
-        :param states: the name of each state (by default, an increasing sequence of integers starting at 1).
+        :param states: the name of each state (if omitted, an increasing sequence of integers starting at 1).
         :raises ValidationError: if any input argument is not compliant.
         """
 
@@ -1870,7 +1901,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         m = _np.interp(m, (_np.min(m), _np.max(m)), (0, 1))
         m = m / _np.sum(m, axis=1, keepdims=True)
@@ -1884,7 +1915,7 @@ class MarkovChain(object):
         The method generates a Markov chain of given size based on an identity transition matrix.
 
         :param size: the size of the chain.
-        :param states: the name of each state (by default, an increasing sequence of integers starting at 1).
+        :param states: the name of each state (if omitted, an increasing sequence of integers starting at 1).
         :return: a Markov chain.
         :raises ValidationError: if any input argument is not compliant.
         """
@@ -1900,7 +1931,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         return MarkovChain(_np.eye(size), states)
 
@@ -1911,7 +1942,7 @@ class MarkovChain(object):
         The method generates a Markov chain of given size with random transition probabilities.
 
         :param size: the size of the chain.
-        :param states: the name of each state (by default, an increasing sequence of integers starting at 1).
+        :param states: the name of each state (if omitted, an increasing sequence of integers starting at 1).
         :param zeros: the number of zero-valued transition probabilities (by default, 0).
         :param mask: a matrix representing the locations and values of fixed transition probabilities.
         :param seed: a seed to be used as RNG initializer for reproducibility purposes.
@@ -1931,7 +1962,7 @@ class MarkovChain(object):
             else:
                 states = _validate_state_names(states, size)
 
-            zeros = _validate_integer_non_negative(zeros)
+            zeros = _validate_integer(zeros, lower_limit=(0, False))
 
             if mask is None:
                 mask = _np.full((size, size), _np.nan, dtype=float)
@@ -1940,7 +1971,7 @@ class MarkovChain(object):
 
         except Exception as e:
             argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-            raise ValidationError(str(e).replace('@arg@', argument)) from None
+            raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
         full_rows = _np.isclose(_np.nansum(mask, axis=1, dtype=float), 1.0)
 
