@@ -39,25 +39,31 @@ from typing import (
     Iterable as _Iterable,
     List as _List,
     Optional as _Optional,
-    Tuple as _Tuple,
     Union as _Union
 )
 
 # Internal
 
+from pydtmc.custom_types import (
+    tnumeric as _tnumeric,
+    oplot as _oplot
+)
+
 from pydtmc.exceptions import (
-    ValidationError
+    ValidationError as _ValidationError
 )
 
 from pydtmc.markov_chain import (
-    MarkovChain
+    MarkovChain as _MarkovChain
 )
 
 from pydtmc.validation import (
     validate_boolean as _validate_boolean,
     validate_distribution as _validate_distribution,
     validate_enumerator as _validate_enumerator,
-    validate_integer_dpi as _validate_integer_dpi,
+    validate_dpi as _validate_dpi,
+    validate_state as _validate_state,
+    validate_status as _validate_status,
     validate_walk as _validate_walk
 )
 
@@ -78,25 +84,27 @@ _colors = ['#80B1D3', '#FFED6F', '#B3DE69', '#BEBADA', '#FDB462', '#8DD3C7', '#F
 #############
 
 
-def plot_eigenvalues(mc: MarkovChain, dpi: int = 100) -> _Optional[_Tuple[_mp.Figure, _mp.Axes]]:
+def plot_eigenvalues(mc: _MarkovChain, dpi: int = 100) -> _oplot:
 
     """
     The function plots the eigenvalues of the Markov chain on the complex plane.
 
     :param mc: the target Markov chain.
     :param dpi: the resolution of the plot expressed in dots per inch (by default, 100).
-    :return: None if Matplotlib is in interactive mode as the plot is immediately displayed, the handles of the plot otherwise.
+    :return: None if Matplotlib is in interactive mode as the plot is immediately displayed, otherwise the handles of the plot.
     :raises ValidationError: if any input argument is not compliant.
     """
 
-    if not isinstance(mc, MarkovChain):
-        raise ValidationError('A valid MarkovChain instance must be provided.')
+    if not isinstance(mc, _MarkovChain):
+        raise ValueError('A valid MarkovChain instance must be provided.')
 
     try:
-        dpi = _validate_integer_dpi(dpi)
+
+        dpi = _validate_dpi(dpi)
+
     except Exception as e:
         argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-        raise ValidationError(str(e).replace('@arg@', argument)) from None
+        raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
     figure, ax = _mp.subplots(dpi=dpi)
 
@@ -165,22 +173,20 @@ def plot_eigenvalues(mc: MarkovChain, dpi: int = 100) -> _Optional[_Tuple[_mp.Fi
     return figure, ax
 
 
-def plot_graph(mc: MarkovChain, nodes_color: bool = True, nodes_type: bool = True, edges_color: bool = True, edges_value: bool = True, dpi: int = 100) -> _Optional[_Tuple[_mp.Figure, _mp.Axes]]:
+def plot_graph(mc: _MarkovChain, nodes_color: bool = True, nodes_type: bool = True, edges_color: bool = True, edges_value: bool = True, dpi: int = 100) -> _oplot:
 
     """
     The function plots the directed graph of the Markov chain.
 
-    | **Notes:** Graphviz and Pydot are required.
+    | **Notes:** Graphviz and Pydot are not required, but they provide access to extended graphs with additional features.
 
     :param mc: the target Markov chain.
     :param nodes_color: a boolean indicating whether to display colored nodes based on communicating classes (by default, True).
     :param nodes_type: a boolean indicating whether to use a different shape for every node type (by default, True).
-    :param edges_color: a boolean indicating whether to display colored edges based on transition probabilities (by default, True).
+    :param edges_color: a boolean indicating whether to display edges using a gradient based on transition probabilities, valid only for extended graphs (by default, True).
     :param edges_value: a boolean indicating whether to display the transition probability of every edge (by default, True).
     :param dpi: the resolution of the plot expressed in dots per inch (by default, 100).
-    :return: None if Matplotlib is in interactive mode as the plot is immediately displayed, the handles of the plot otherwise.
-    :raises EnvironmentError: if Graphviz is not installed.
-    :raises ImportError: if Pydot is not installed.
+    :return: None if Matplotlib is in interactive mode as the plot is immediately displayed, otherwise the handles of the plot.
     :raises ValidationError: if any input argument is not compliant.
     """
 
@@ -214,18 +220,8 @@ def plot_graph(mc: MarkovChain, nodes_color: bool = True, nodes_type: bool = Tru
 
         return clist
 
-    try:
-        _call(['dot', '-V'], stdout=_PIPE, stderr=_PIPE)
-    except Exception:
-        raise EnvironmentError('Graphviz is required by this plotting function.') from None
-
-    try:
-        import pydot as pyd
-    except ImportError:
-        raise ImportError('Pydot is required by this plotting function.') from None
-
-    if not isinstance(mc, MarkovChain):
-        raise ValidationError('A valid MarkovChain instance must be provided.')
+    if not isinstance(mc, _MarkovChain):
+        raise ValueError('A valid MarkovChain instance must be provided.')
 
     try:
 
@@ -233,128 +229,184 @@ def plot_graph(mc: MarkovChain, nodes_color: bool = True, nodes_type: bool = Tru
         nodes_type = _validate_boolean(nodes_type)
         edges_color = _validate_boolean(edges_color)
         edges_value = _validate_boolean(edges_value)
-        dpi = _validate_integer_dpi(dpi)
+        dpi = _validate_dpi(dpi)
 
     except Exception as e:
         argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-        raise ValidationError(str(e).replace('@arg@', argument)) from None
+        raise _ValidationError(str(e).replace('@arg@', argument)) from None
+
+    extended_graph = True
+
+    # noinspection PyBroadException
+    try:
+        _call(['dot', '-V'], stdout=_PIPE, stderr=_PIPE)
+    except Exception:
+        extended_graph = False
+        pass
+
+    try:
+        import pydot as pyd
+    except ImportError:
+        extended_graph = False
+        pass
 
     g = mc.to_directed_graph()
-    g_pydot = _nx.nx_pydot.to_pydot(g)
 
-    if nodes_color:
-        c = node_colors(len(mc.communicating_classes))
-        for node in g_pydot.get_nodes():
-            state = node.get_name()
-            for x, cc in enumerate(mc.communicating_classes):
-                if state in cc:
-                    node.set_style('filled')
-                    node.set_fillcolor(c[x])
-                    break
+    if extended_graph:
 
-    if nodes_type:
-        for node in g_pydot.get_nodes():
-            if node.get_name() in mc.transient_states:
-                node.set_shape('box')
+        g_pydot = _nx.nx_pydot.to_pydot(g)
+
+        if nodes_color:
+            c = node_colors(len(mc.communicating_classes))
+            for node in g_pydot.get_nodes():
+                state = node.get_name()
+                for x, cc in enumerate(mc.communicating_classes):
+                    if state in cc:
+                        node.set_style('filled')
+                        node.set_fillcolor(c[x])
+                        break
+
+        if nodes_type:
+            for node in g_pydot.get_nodes():
+                if node.get_name() in mc.transient_states:
+                    node.set_shape('box')
+                else:
+                    node.set_shape('ellipse')
+
+        if edges_color:
+            c = edge_colors(_color_gray, _color_black, 20)
+            for edge in g_pydot.get_edges():
+                probability = mc.transition_probability(edge.get_destination(), edge.get_source())
+                x = int(round(probability * 20.0)) - 1
+                edge.set_style('filled')
+                edge.set_color(c[x])
+
+        if edges_value:
+            for edge in g_pydot.get_edges():
+                probability = mc.transition_probability(edge.get_destination(), edge.get_source())
+                if probability.is_integer():
+                    edge.set_label(f' {probability:g}.0 ')
+                else:
+                    edge.set_label(f' {round(probability,2):g} ')
+
+        buffer = _BytesIO()
+        buffer.write(g_pydot.create_png())
+        buffer.seek(0)
+
+        img = _mi.imread(buffer)
+        img_x = img.shape[0] / dpi
+        img_y = img.shape[1] / dpi
+
+        figure = _mp.figure(figsize=(img_y, img_x), dpi=dpi)
+        figure.figimage(img)
+        ax = figure.gca()
+
+    else:
+
+        mpi = _mp.isinteractive()
+        _mp.interactive(False)
+
+        figure, ax = _mp.subplots(dpi=dpi)
+
+        positions = _nx.spring_layout(g)
+        node_colors_all = node_colors(len(mc.communicating_classes))
+
+        for node in g.nodes:
+
+            node_color = None
+
+            if nodes_color:
+                for x, cc in enumerate(mc.communicating_classes):
+                    if node in cc:
+                        node_color = node_colors_all[x]
+                        break
+
+            if nodes_type:
+                if node in mc.transient_states:
+                    node_shape = 's'
+                else:
+                    node_shape = 'o'
             else:
-                node.set_shape('ellipse')
+                node_shape = None
 
-    if edges_color:
-        c = edge_colors(_color_gray, _color_black, 20)
-        for edge in g_pydot.get_edges():
-            probability = mc.transition_probability(edge.get_destination(), edge.get_source())
-            x = int(round(probability * 20.0)) - 1
-            edge.set_style('filled')
-            edge.set_color(c[x])
-
-    if edges_value:
-        for edge in g_pydot.get_edges():
-            probability = mc.transition_probability(edge.get_destination(), edge.get_source())
-            if probability.is_integer():
-                edge.set_label(f' {probability:g}.0 ')
+            if node_color is not None and node_shape is not None:
+                _nx.draw_networkx_nodes(g, positions, ax=ax, nodelist=[node], edgecolors='k', node_color=node_color, node_shape=node_shape)
+            elif node_color is not None and node_shape is None:
+                _nx.draw_networkx_nodes(g, positions, ax=ax, nodelist=[node], edgecolors='k', node_color=node_color)
+            elif node_color is None and node_shape is not None:
+                _nx.draw_networkx_nodes(g, positions, ax=ax, nodelist=[node], edgecolors='k', node_shape=node_shape)
             else:
-                edge.set_label(f' {round(probability,2):g} ')
+                _nx.draw_networkx_nodes(g, positions, ax=ax, edgecolors='k')
 
-    buffer = _BytesIO()
-    buffer.write(g_pydot.create_png())
-    buffer.seek(0)
+        _nx.draw_networkx_labels(g, positions, ax=ax)
 
-    img = _mi.imread(buffer)
-    img_x = img.shape[0] / dpi
-    img_y = img.shape[1] / dpi
+        _nx.draw_networkx_edges(g, positions, ax=ax, arrows=False)
 
-    figure = _mp.figure(figsize=(img_y, img_x), dpi=dpi)
-    figure.figimage(img)
+        if edges_value:
+
+            edges_values = dict()
+
+            for edge in g.edges:
+                probability = mc.transition_probability(edge[1], edge[0])
+                if probability.is_integer():
+                    value = f' {probability:g}.0 '
+                else:
+                    value = f' {round(probability,2):g} '
+                edges_values[(edge[0], edge[1])] = value
+
+            _nx.draw_networkx_edge_labels(g, positions, ax=ax, edge_labels=edges_values, label_pos=0.7)
+
+        _mp.interactive(mpi)
 
     if _mp.isinteractive():
         _mp.show(block=False)
         return None
 
-    return figure, figure.gca()
+    return figure, ax
 
 
-def plot_redistributions(mc: MarkovChain, distributions: _Union[int, _Iterable[_np.ndarray]], plot_type: str = 'curves', dpi: int = 100) -> _Optional[_Tuple[_mp.Figure, _mp.Axes]]:
+def plot_redistributions(mc: _MarkovChain, distributions: _Union[int, _Iterable[_np.ndarray]], initial_status: _Optional[_Union[int, str, _tnumeric]] = None, plot_type: str = 'projection', dpi: int = 100) -> _oplot:
 
     """
     The function plots a redistribution of states on the given Markov chain.
 
     :param mc: the target Markov chain.
     :param distributions: a sequence of redistributions or the number of redistributions to perform.
-    :param plot_type: the type of plot to display (either curves or heatmap, curves by default).
+    :param initial_status: the initial state or the initial distribution of the states (if omitted, the states are assumed to be uniformly distributed).
+    :param plot_type: the type of plot to display (either heatmap or projection; projection by default).
     :param dpi: the resolution of the plot expressed in dots per inch (by default, 100).
-    :return: None if Matplotlib is in interactive mode as the plot is immediately displayed, the handles of the plot otherwise.
+    :return: None if Matplotlib is in interactive mode as the plot is immediately displayed, otherwise the handles of the plot.
     :raises ValidationError: if any input argument is not compliant.
     """
 
-    if not isinstance(mc, MarkovChain):
-        raise ValidationError('A valid MarkovChain instance must be provided.')
+    if not isinstance(mc, _MarkovChain):
+        raise ValueError('A valid MarkovChain instance must be provided.')
 
     try:
 
         distributions = _validate_distribution(distributions, mc.size)
-        plot_type = _validate_enumerator(plot_type, ['curves', 'heatmap'])
-        dpi = _validate_integer_dpi(dpi)
+
+        if initial_status is not None:
+            initial_status = _validate_status(initial_status, mc.states)
+
+        plot_type = _validate_enumerator(plot_type, ['heatmap', 'projection'])
+        dpi = _validate_dpi(dpi)
 
     except Exception as e:
         argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-        raise ValidationError(str(e).replace('@arg@', argument)) from None
+        raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
     if isinstance(distributions, int):
-        distributions = mc.redistribute(distributions, include_initial=True)
+        distributions = mc.redistribute(distributions, initial_status=initial_status, include_initial=True, output_last=False)
+    elif initial_status is not None and not _np.array_equal(distributions[0], initial_status):
+        raise ValueError('The "initial_status" parameter, if specified when the "distributions" parameter represents a sequence of redistributions, must match the first element.')
 
     distribution_len = len(distributions)
     distributions = _np.array(distributions)
 
     figure, ax = _mp.subplots(dpi=dpi)
 
-    if plot_type == 'curves':
-
-        ax.set_prop_cycle('color', _colors)
-
-        for i in range(mc.size):
-            ax.plot(_np.arange(0.0, distribution_len, 1.0), distributions[:, i], label=mc.states[i], marker='o')
-
-        if _np.array_equal(distributions[0, :], _np.ones(mc.size, dtype=float) / mc.size):
-            ax.plot(0.0, distributions[0, 0], color=_color_black, label="Start", marker='o', markeredgecolor=_color_black, markerfacecolor=_color_black)
-            legend_size = mc.size + 1
-        else:
-            legend_size = mc.size
-
-        ax.set_xlabel('Steps', fontsize=13.0)
-        ax.set_xticks(_np.arange(0.0, distribution_len, 1.0 if distribution_len <= 11 else 10.0))
-        ax.set_xticklabels(_np.arange(0, distribution_len, 1 if distribution_len <= 11 else 10))
-        ax.set_xlim(-0.5, distribution_len - 0.5)
-
-        ax.set_ylabel('Frequencies', fontsize=13.0)
-        ax.set_yticks(_np.linspace(0.0, 1.0, 11))
-        ax.set_ylim(0.0, 1.0)
-
-        ax.legend(bbox_to_anchor=(0.5, -0.15), loc='upper center', ncol=legend_size)
-        ax.set_title('Redistplot (Curves)', fontsize=15.0, fontweight='bold')
-
-        _mp.subplots_adjust(bottom=0.2)
-
-    else:
+    if plot_type == 'heatmap':
 
         color_map = _mc.LinearSegmentedColormap.from_list('ColorMap', [_color_white, _colors[0]], 20)
         ax_is = ax.imshow(_np.transpose(distributions), aspect='auto', cmap=color_map, interpolation='none', vmin=0.0, vmax=1.0)
@@ -376,6 +428,38 @@ def plot_redistributions(mc: MarkovChain, distributions: _Union[int, _Iterable[_
 
         ax.set_title('Redistplot (Heatmap)', fontsize=15.0, fontweight='bold')
 
+    else:
+
+        ax.set_prop_cycle('color', _colors)
+
+        if distribution_len == 2:
+            for i in range(mc.size):
+                ax.plot(_np.arange(0.0, distribution_len, 1.0), distributions[:, i], label=mc.states[i], marker='o')
+        else:
+            for i in range(mc.size):
+                ax.plot(_np.arange(0.0, distribution_len, 1.0), distributions[:, i], label=mc.states[i])
+
+        if _np.array_equal(distributions[0, :], _np.ones(mc.size, dtype=float) / mc.size):
+            ax.plot(0.0, distributions[0, 0], color=_color_black, label="Start", marker='o', markeredgecolor=_color_black, markerfacecolor=_color_black)
+            legend_size = mc.size + 1
+        else:
+            legend_size = mc.size
+
+        ax.set_xlabel('Steps', fontsize=13.0)
+        ax.set_xticks(_np.arange(0.0, distribution_len, 1.0 if distribution_len <= 11 else 10.0))
+        ax.set_xticklabels(_np.arange(0, distribution_len, 1 if distribution_len <= 11 else 10))
+        ax.set_xlim(-1.0 * (distribution_len * 0.05), distribution_len * 1.05)
+
+        ax.set_ylabel('Frequencies', fontsize=13.0)
+        ax.set_yticks(_np.linspace(0.0, 1.0, 11))
+        ax.set_ylim(-0.05, 1.05)
+
+        ax.grid()
+        ax.legend(bbox_to_anchor=(0.5, -0.15), loc='upper center', ncol=legend_size)
+        ax.set_title('Redistplot (Projection)', fontsize=15.0, fontweight='bold')
+
+        _mp.subplots_adjust(bottom=0.2)
+
     if _mp.isinteractive():
         _mp.show(block=False)
         return None
@@ -383,34 +467,41 @@ def plot_redistributions(mc: MarkovChain, distributions: _Union[int, _Iterable[_
     return figure, ax
 
 
-def plot_walk(mc: MarkovChain, walk: _Union[int, _Iterable[int], _Iterable[str]], plot_type: str = 'histogram', dpi: int = 100) -> _Optional[_Tuple[_mp.Figure, _mp.Axes]]:
+def plot_walk(mc: _MarkovChain, walk: _Union[int, _Iterable[int], _Iterable[str]], initial_state: _Optional[_Union[int, str]] = None, plot_type: str = 'histogram', dpi: int = 100) -> _oplot:
 
     """
     The function plots a random walk on the given Markov chain.
 
     :param mc: the target Markov chain.
     :param walk: a sequence of states or the number of simulations to perform.
-    :param plot_type: the type of plot to display (either histogram, sequence or transitions, histogram by default).
+    :param initial_state: the initial state of the walk (if omitted, it is chosen uniformly at random).
+    :param plot_type: the type of plot to display (one of histogram, sequence and transitions; histogram by default).
     :param dpi: the resolution of the plot expressed in dots per inch (by default, 100).
-    :return: None if Matplotlib is in interactive mode as the plot is immediately displayed, the handles of the plot otherwise.
+    :return: None if Matplotlib is in interactive mode as the plot is immediately displayed, otherwise the handles of the plot.
     :raises ValidationError: if any input argument is not compliant.
     """
 
-    if not isinstance(mc, MarkovChain):
-        raise ValidationError('A valid MarkovChain instance must be provided.')
+    if not isinstance(mc, _MarkovChain):
+        raise ValueError('A valid MarkovChain instance must be provided.')
 
     try:
 
         walk = _validate_walk(walk, mc.states)
+
+        if initial_state is not None:
+            initial_state = _validate_state(initial_state, mc.states)
+
         plot_type = _validate_enumerator(plot_type, ['histogram', 'sequence', 'transitions'])
-        dpi = _validate_integer_dpi(dpi)
+        dpi = _validate_dpi(dpi)
 
     except Exception as e:
         argument = ''.join(_trace()[0][4]).split('=', 1)[0].strip()
-        raise ValidationError(str(e).replace('@arg@', argument)) from None
+        raise _ValidationError(str(e).replace('@arg@', argument)) from None
 
     if isinstance(walk, int):
-        walk = mc.walk(walk, include_initial=True, output_indices=True)
+        walk = mc.walk(walk, initial_state=initial_state, include_initial=True, output_indices=True)
+    elif initial_state is not None and (walk[0] != initial_state):
+        raise ValueError('The "initial_state" parameter, if specified when the "walk" parameter represents a sequence of states, must match the first element.')
 
     walk_len = len(walk)
 
