@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 
 __all__ = [
-    'validate_boolean', 'validate_enumerator', 'validate_integer_dpi', 'validate_integer_non_negative', 'validate_integer_positive',
-    'validate_mask', 'validate_matrix', 'validate_transition_matrix', 'validate_transition_matrix_size',
+    'validate_boolean', 'validate_enumerator', 'validate_dpi', 'validate_integer',
+    'validate_dictionary', 'validate_mask', 'validate_matrix', 'validate_transition_matrix', 'validate_transition_matrix_size',
     'validate_distribution', 'validate_hyperparameter', 'validate_rewards', 'validate_vector', 'validate_walk',
-    'validate_state', 'validate_states', 'validate_state_names'
+    'validate_state', 'validate_states', 'validate_state_names', 'validate_status'
 ]
 
 
@@ -19,18 +19,21 @@ import numpy as _np
 
 # Minor
 
-from collections.abc import (
-    Iterable as _CIterable
-)
-
 from copy import (
     deepcopy as _deepcopy
 )
 
+from numbers import (
+    Number as _Number
+)
+
 from typing import (
     Any as _Any,
+    Dict as _Dict,
+    Iterable as _Iterable,
     List as _List,
     Optional as _Optional,
+    Tuple as _Tuple,
     Union as _Union
 )
 
@@ -52,53 +55,56 @@ except ImportError:
 #############
 
 
-def extract_non_numeric(data: _Any) -> list:
+def extract_non_numeric(data: _Any) -> _List[_Any]:
 
-    if isinstance(data, list):
-        return _deepcopy(data)
+    result = None
 
-    if isinstance(data, _CIterable) and not isinstance(data, str):
-        return list(data)
+    if isinstance(data, _Dict):
+        result = list(data.values())
+    elif isinstance(data, _List):
+        result = _deepcopy(data)
+    elif isinstance(data, _Iterable) and not isinstance(data, str):
+        result = list(data)
 
-    raise TypeError('The data type is not supported.')
+    if result is None or any(isinstance(x, _Number) for x in result):
+        raise TypeError('The data type is not supported.')
+
+    return result
 
 
 def extract_numeric(data: _Any) -> _np.ndarray:
 
+    result = None
+
     if isinstance(data, list):
-        return _np.array(data)
+        result = _np.array(data)
+    elif isinstance(data, _np.ndarray):
+        result = _np.copy(data)
+    elif _pd and isinstance(data, (_pd.DataFrame, _pd.Series)):
+        result = data.to_numpy(copy=True)
+    elif _sps and isinstance(data, _sps.bsr.bsr_matrix):
+        result = _np.array(data.todense())
+    elif _sps and isinstance(data, _sps.coo.coo_matrix):
+        result = _np.array(data.todense())
+    elif _sps and isinstance(data, _sps.csc.csc_matrix):
+        result = _np.array(data.todense())
+    elif _sps and isinstance(data, _sps.csr.csr_matrix):
+        result = _np.array(data.todense())
+    elif _sps and isinstance(data, _sps.dia.dia_matrix):
+        result = _np.array(data.todense())
+    elif _sps and isinstance(data, _sps.dok.dok_matrix):
+        result = _np.array(data.todense())
+    elif _sps and isinstance(data, _sps.lil.lil_matrix):
+        result = _np.array(data.todense())
+    elif isinstance(data, _Dict):
+        result = _np.array(list(data.values()))
+    elif isinstance(data, _Iterable) and not isinstance(data, str):
+        result = _np.array(list(data))
 
-    if isinstance(data, _np.ndarray):
-        return data.copy()
+    if result is None or not _np.issubdtype(result.dtype, _np.number):
+        raise TypeError('The data type is not supported.')
 
-    if _pd and isinstance(data, (_pd.DataFrame, _pd.Series)):
-        return data.values.copy()
-
-    if _sps and isinstance(data, _sps.bsr.bsr_matrix):
-        return _np.array(data.todense())
-
-    if _sps and isinstance(data, _sps.coo.coo_matrix):
-        return _np.array(data.todense())
-
-    if _sps and isinstance(data, _sps.csc.csc_matrix):
-        return _np.array(data.todense())
-
-    if _sps and isinstance(data, _sps.csr.csr_matrix):
-        return _np.array(data.todense())
-
-    if _sps and isinstance(data, _sps.dia.dia_matrix):
-        return _np.array(data.todense())
-
-    if _sps and isinstance(data, _sps.dok.dok_matrix):
-        return _np.array(data.todense())
-
-    if _sps and isinstance(data, _sps.lil.lil_matrix):
-        return _np.array(data.todense())
-
-    if isinstance(data, _CIterable):
-        return _np.array(list(data))
-
-    raise TypeError('The data type is not supported.')
+    return result
 
 
 def validate_boolean(value: _Any) -> bool:
@@ -114,17 +120,25 @@ def validate_distribution(distribution: _Any, size: int) -> _Union[int, _List[_n
     if isinstance(distribution, int):
 
         if distribution <= 0:
-            raise ValueError('The "@arg@" parameter must be positive.')
+            raise ValueError('The "@arg@" parameter, when specified as an integer, must be greater than or equal to 1.')
 
         return distribution
 
     elif isinstance(distribution, list):
 
+        distribution_len = len(distribution)
+
+        if distribution_len == 0:
+            raise ValueError('The "@arg@" parameter, when specified as a list of vectors, must be non-empty.')
+
         for i, vector in distribution:
 
             if not isinstance(vector, _np.ndarray) or not _np.issubdtype(vector.dtype, _np.number):
                 raise TypeError('The "@arg@" parameter must contain only numeric vectors.')
 
+            if distribution_len <= 1:
+                raise ValueError('The "@arg@" parameter, when specified as a list of vectors, must contain at least 2 elements.')
+
             vector = vector.astype(float)
             distribution[i] = vector
 
@@ -178,7 +192,40 @@ def validate_hyperparameter(hyperparameter: _Any, size: int) -> _np.ndarray:
     return hyperparameter
 
 
-def validate_integer_dpi(value: _Any) -> int:
+def validate_dictionary(d: _Any) -> _Dict[_Tuple[str, str], float]:
+
+    if not isinstance(d, dict):
+        raise ValueError('The "@arg@" parameter must be a dictionary.')
+
+    keys = d.keys()
+
+    if not all(isinstance(key, tuple) and (len(key) == 2) and isinstance(key[0], str) and isinstance(key[1], str) for key in keys):
+        raise ValueError('The "@arg@" parameter keys must be tuples containing 2 string values.')
+
+    keys = sorted(list(set(sum(keys, ()))))
+
+    if not all(key is not None and (len(key) > 0) for key in keys):
+        raise TypeError('The "@arg@" parameter keys must contain only valid string values.')
+
+    values = d.values()
+
+    if not all(isinstance(value, (float, int)) for value in values):
+        raise ValueError('The "@arg@" parameter values must be float or integer numbers.')
+
+    values = [float(value) for value in values]
+
+    if not all((value >= 0.0) and (value <= 1.0) for value in values):
+        raise ValueError('The "@arg@" parameter values can contain only numbers between 0 and 1.')
+
+    result = {}
+
+    for key, value in d.items():
+        result[key] = float(value)
+
+    return result
+
+
+def validate_dpi(value: _Any) -> int:
 
     if not isinstance(value, int):
         raise TypeError('The "@arg@" parameter must be an integer value.')
@@ -189,24 +236,26 @@ def validate_integer_dpi(value: _Any) -> int:
     return value
 
 
-def validate_integer_non_negative(value: _Any) -> int:
-
-    if not isinstance(value, int):
-        raise TypeError('The "@arg@" parameter must an integer value.')
-
-    if value < 0:
-        raise ValueError('The "@arg@" parameter must be non-negative.')
-
-    return value
-
-
-def validate_integer_positive(value: _Any) -> int:
+def validate_integer(value: _Any, lower_limit: _Tuple[int, bool] = None, upper_limit: _Tuple[int, bool] = None) -> int:
 
     if not isinstance(value, int):
         raise TypeError('The "@arg@" parameter must be an integer value.')
 
-    if value <= 0:
-        raise ValueError('The "@arg@" parameter must be positive.')
+    if lower_limit is not None:
+        if lower_limit[1]:
+            if value <= lower_limit[0]:
+                raise ValueError(f'The "@arg@" parameter must be greater than {lower_limit[0]:d}.')
+        else:
+            if value < lower_limit[0]:
+                raise ValueError(f'The "@arg@" parameter must be greater than or equal to {lower_limit[0]:d}.')
+
+    if upper_limit is not None:
+        if upper_limit[1]:
+            if value >= upper_limit[0]:
+                raise ValueError(f'The "@arg@" parameter must be less than {upper_limit[0]:d}.')
+        else:
+            if value > upper_limit[0]:
+                raise ValueError(f'The "@arg@" parameter must be less than or equal to {upper_limit[0]:d}.')
 
     return value
 
@@ -306,6 +355,61 @@ def validate_state(state: _Any, current_states: list) -> int:
     raise TypeError('The "@arg@" parameter must be either an integer or a string.')
 
 
+def validate_status(status: _Any, current_states: list) -> _np.ndarray:
+
+    size = len(current_states)
+
+    if isinstance(status, int):
+
+        limit = size - 1
+
+        if (status < 0) or (status > limit):
+            raise ValueError(f'The "@arg@" parameter, when specified as an integer, must have a value between 0 and the number of existing states minus one ({limit:d}).')
+
+        result = _np.zeros(size, dtype=float)
+        result[status] = 1.0
+
+        return result
+
+    if isinstance(status, str):
+
+        if status not in current_states:
+            raise ValueError(f'The "@arg@" parameter, when specified as a string, must match the name of an existing state ({", ".join(current_states)}).')
+
+        status = current_states.index(status)
+
+        result = _np.zeros(size, dtype=float)
+        result[status] = 1.0
+
+        return result
+
+    try:
+        status = extract_numeric(status)
+    except Exception:
+        raise TypeError('The "@arg@" parameter is null or wrongly typed.')
+
+    if not _np.issubdtype(status.dtype, _np.number):
+        raise TypeError('The "@arg@" parameter must contain only numeric values.')
+
+    status = status.astype(float)
+
+    if (status.ndim < 1) or ((status.ndim == 2) and (status.shape[0] != 1) and (status.shape[1] != 1)) or (status.ndim > 2):
+        raise ValueError('The "@arg@" parameter must be a valid vector.')
+
+    status = _np.ravel(status)
+
+    if status.size != size:
+        raise ValueError(f'The "@arg@" parameter length must be equal to the number of states ({size:d}).')
+
+    if not all(_np.isfinite(x) and (x >= 0.0) and (x <= 1.0) for x in _np.nditer(status)):
+        raise ValueError('The "@arg@" parameter must contain only values between 0 and 1.')
+
+    if not _np.isclose(_np.sum(status), 1.0):
+        raise ValueError('The "@arg@" parameter values must sum to 1.')
+
+    return status
+
+
 def validate_state_names(states: _Any, size: _Optional[int] = None) -> _List[str]:
 
     try:
@@ -316,6 +420,9 @@ def validate_state_names(states: _Any, size: _Optional[int] = None) -> _List[str
     if not all(isinstance(s, str) for s in states):
         raise TypeError('The "@arg@" parameter must contain only string values.')
 
+    if not all(s is not None and (len(s) > 0) for s in states):
+        raise TypeError('The "@arg@" parameter must contain only valid string values.')
+
     states_length = len(states)
     states_unique = len(set(states))
 
@@ -323,14 +430,14 @@ def validate_state_names(states: _Any, size: _Optional[int] = None) -> _List[str
         raise ValueError('The "@arg@" parameter must contain only unique values.')
 
     if size is not None and (states_length != size):
-        raise ValueError(f'The "@arg@" parameter must contain a number of elements equal {size:d}.')
+        raise ValueError(f'The "@arg@" parameter must contain a number of elements equal to {size:d}.')
 
     return states
 
 
-def validate_states(states: _Any, current_states: _List[str], states_type: str, states_flex: bool) -> _List[int]:
+def validate_states(states: _Any, current_states: _List[str], state_type: str, flex: bool) -> _List[int]:
 
-    if states_flex:
+    if flex:
 
         if isinstance(states, int):
 
@@ -369,20 +476,20 @@ def validate_states(states: _Any, current_states: _List[str], states_type: str,
 
     else:
 
-        if states_flex:
+        if flex:
             raise TypeError('The "@arg@" parameter must be either an integer, a string, an array_like object of integers or an array_like object of strings.')
         else:
             raise TypeError('The "@arg@" parameter must be either an array_like object of integers or an array_like object of strings.')
 
     states_length = len(states)
 
-    if (states_type != 'walk') and (len(set(states)) < states_length):
+    if (state_type != 'walk') and (len(set(states)) < states_length):
         raise ValueError('The "@arg@" parameter must contain only unique values.')
 
-    if states_type == 'regular':
+    if state_type == 'regular':
         if (states_length < 1) or (states_length > current_states_length):
             raise ValueError(f'The "@arg@" parameter must contain a number of elements between 1 and the number of existing states ({current_states_length:d}).')
-    elif states_type == 'subset':
+    elif state_type == 'subset':
         if (states_length < 1) or (states_length >= current_states_length):
             raise ValueError(f'The "@arg@" parameter must contain a number of elements between 1 and the number of existing states minus one ({current_states_length - 1:d}).')
     else:
@@ -432,32 +539,25 @@ def validate_transition_matrix_size(size: _Any) -> int:
     return size
 
 
-def validate_vector(vector: _Any, vtype: str, flex: _Optional[bool] = None, size: _Optional[int] = None) -> _np.ndarray:
+def validate_vector(vector: _Any, vector_type: str, flex: bool, size: _Optional[int] = None) -> _np.ndarray:
 
-    if flex:
-
-        if isinstance(vector, int):
-            vector = float(vector)
-
-        if isinstance(vector, float):
-
-            if not _np.isfinite(vector) or (vector < 0.0) or (vector > 1.0):
-                raise ValueError(f'The "@arg@" parameter, when specified as a numeric scalar, must have a value between 0 and 1.')
-
-            return _np.repeat(vector, size)
-
-    try:
-        vector = extract_numeric(vector)
-    except Exception:
-        raise TypeError('The "@arg@" parameter is null or wrongly typed.')
+    if flex and size is not None and isinstance(vector, int):
+        vector = _np.repeat(float(vector), size)
+    elif flex and size is not None and isinstance(vector, float):
+        vector = _np.repeat(vector, size)
+    else:
+        try:
+            vector = extract_numeric(vector)
+        except Exception:
+            raise TypeError('The "@arg@" parameter is null or wrongly typed.')
 
     if not _np.issubdtype(vector.dtype, _np.number):
         raise TypeError('The "@arg@" parameter must contain only numeric values.')
 
     vector = vector.astype(float)
 
-    if (vector.ndim < 1) or ((vector.ndim == 2) and (vector.shape[0] != 1)) or (vector.ndim > 2):
-        raise ValueError('The "@arg@" parameter must be a vector.')
+    if (vector.ndim < 1) or ((vector.ndim == 2) and (vector.shape[0] != 1) and (vector.shape[1] != 1)) or (vector.ndim > 2):
+        raise ValueError('The "@arg@" parameter must be a valid vector.')
 
     vector = _np.ravel(vector)
 
@@ -467,20 +567,20 @@ def validate_vector(vector: _Any, vtype: str, flex: _Optional[bool] = None, size
     if not all(_np.isfinite(x) and (x >= 0.0) and (x <= 1.0) for x in _np.nditer(vector)):
         raise ValueError('The "@arg@" parameter must contain only values between 0 and 1.')
 
-    if vtype == 'annihilation':
+    if vector_type == 'annihilation':
         if not _np.isclose(vector[0], 0.0):
             raise ValueError('The "@arg@" parameter must contain a value equal to 0 in the first index.')
-    elif vtype == 'creation':
+    elif vector_type == 'creation':
         if not _np.isclose(vector[-1], 0.0):
             raise ValueError('The "@arg@" parameter must contain a value equal to 0 in the last index.')
-    elif vtype == 'stochastic':
+    elif vector_type == 'stochastic':
         if not _np.isclose(_np.sum(vector), 1.0):
             raise ValueError('The "@arg@" parameter values must sum to 1.')
 
     return vector
 
 
-def validate_walk(walk: _Any, current_states: _List[str]) -> _Union[int, _Union[_List[int], _List[str]]]:
+def validate_walk(walk: _Any, current_states: _List[str]) -> _Union[int, _List[int], _List[str]]:
 
     if isinstance(walk, int):
 
